import mongoose, { Model, Types } from 'mongoose';
import { Blog } from './schema/blog.schema';
import { CreateBlogDto } from './dto/create-blog.dto';
import { UpdateBlogDto } from './dto/update-blog.dto';

export class BlogRepository<BlogDocument extends Blog> {
    constructor(private readonly model: Model<BlogDocument>) { }

    async createEntity(data: CreateBlogDto): Promise<Blog> {
        const createdEntity = new this.model({
            ...data,
            _id: new Types.ObjectId()
        });
        return await createdEntity.save();
    }

    async findOneEntity(id: string): Promise<Blog | null> {
        if (!mongoose.Types.ObjectId.isValid(id)) {
            return null;
        }
        
        return await this.model.findOne({ _id: id, isDeleted: false });
    }

    async findOneEntityBySlug(urlSlug: string): Promise<Blog | null> {
        try {
            return await this.model.findOne({ urlSlug: urlSlug, isDeleted: false });
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, data: UpdateBlogDto): Promise<Blog | null> {
        return await this.model.findByIdAndUpdate(id, data, { new: true });
    }

    async paginate(page: number, limit: number, query?: any) {
        const skip = (page - 1) * limit;
        const total = await this.model.countDocuments({ ...query, isDeleted: false });
        const pageCount = Math.ceil(total / limit);

        const data = await this.model
            .find({ ...query, isDeleted: false })
            .skip(skip)
            .limit(limit)
            .sort({ createdAt: -1 })

        return {
            data: data,
            pageCount: pageCount,
            total: total
        };
    }

    async deleteEntity(id: string): Promise<boolean> {
        const data = await this.model.findByIdAndUpdate(
            id,
            { isDeleted: true },
            { new: true }
        );
        if (data) return true;
        return false;
    }

    async findAll(): Promise<Blog[]> {
        return await this.model.find({ isDeleted: false });
    }

    async findOneByFilterQuery(query: any): Promise<Blog | null> {
        return await this.model.find({ ...query, isDeleted: false }).lean();
    }

    async countAllQuery(): Promise<any | Error> {
        try {
            return await this.model.countDocuments({
                isDeleted: false
            });
        } catch (e) {
            return e;
        }
    }
}