import { BadRequestException, Injectable, NotFoundException } from '@nestjs/common';
import { InjectModel } from '@nestjs/mongoose';
import { Model } from 'mongoose';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { Blog } from './schema/blog.schema';
import { BlogRepository } from './blog.repository';
import { CreateBlogDto } from './dto/create-blog.dto';
import { UpdateBlogDto } from './dto/update-blog.dto';
import { urlSlug } from 'src/utils/common-helper';

@Injectable()
export class BlogService {
    constructor(
        private readonly doSpaceService: DoSpacesService,
        @InjectModel('blog') private readonly model: Model<Blog>
    ) { }

    private readonly repository = new BlogRepository(this.model);

    async create(
        dto: CreateBlogDto,
        files: {
            image?: UploadedMulterFileI
        }
    ): Promise<Blog> {
        if (files && files.image) {
            const img: any = await this.doSpaceService.uploadFile(files.image[0], "Blog/");
            dto.image = img;
        }

        dto.slug = urlSlug(dto.title)

        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new BadRequestException(
                Constants.CREATE_FAILED
            );
        }
        return ResponseUtils.successResponseHandler(201, "Successfully added to Blog.", "data", data);
    }

    async findAll(pageParam?: string, limitParam?: string): Promise<Blog[]> {
        const page = parseInt(pageParam, 10) ? parseInt(pageParam, 10) : 1;
        const limit = parseInt(limitParam, 10) ? parseInt(limitParam, 10) : 10;
        const data = await this.repository.paginate(page, limit);
        if (!data) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<Blog | null> {
        const data = await this.repository.findOneEntity(id);
        if (!data) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOneByFilterQuery(query: any): Promise<Blog | null> {
        return await this.model.find({ ...query, isDeleted: false }).lean();
    }

    async update(
        id: string, 
        dto: UpdateBlogDto,
        files: {
            image?: UploadedMulterFileI
        }
    ): Promise<Blog | null> {
        const res = await this.repository.findOneEntity(id);
        if (!res) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }
        const foundImage = (res as Blog)?.image;
        if (files && files.image) {
            const img: any = await this.doSpaceService.uploadFile(files.image[0], "Blog/");
            dto.image = img;
        } else {
            dto.image = foundImage;
        }

        dto.slug = urlSlug(dto.title)

        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new BadRequestException(
                Constants.UPDATE_FAILED
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }

    async delete(id: string): Promise<void> {
        const res = await this.repository.findOneEntity(id);
        if (!res) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }

        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new BadRequestException(
                Constants.DELETE_FAILED
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}