import mongoose, { Model, Types } from 'mongoose';
import { CategoryDetails } from './schema/category-details.schema';
import { CreateCategoryDetailsDto } from './dto/create-category-details.dto';
import { UpdateCategoryDetailsDto } from './dto/update-category-details.dto';


export class CategoryDetailsRepository<CategoryDetailsDocument extends CategoryDetails> {
    constructor(private readonly model: Model<CategoryDetailsDocument>) { }

    async createEntity(data: CreateCategoryDetailsDto): Promise<CategoryDetails> {
        const createdEntity = new this.model({
            ...data,
            _id: new Types.ObjectId()
        });
        return await createdEntity.save();
    }

    async findOneEntity(id: string): Promise<CategoryDetails | null> {
        if (!mongoose.Types.ObjectId.isValid(id)) {
            return null;
        }
        
        return await this.model.findOne({ _id: id, isDeleted: false });
    }

    async findOneEntityBySlug(urlSlug: string): Promise<CategoryDetails | null> {
        try {
            return await this.model.findOne({ urlSlug: urlSlug, isDeleted: false });
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, data: UpdateCategoryDetailsDto): Promise<CategoryDetails | null> {
        return await this.model.findByIdAndUpdate(id, data, { new: true });
    }

    async paginate(page: number, limit: number, query?: any) {
        const skip = (page - 1) * limit;
        const total = await this.model.countDocuments({ ...query, isDeleted: false });
        const pageCount = Math.ceil(total / limit);

        const data = await this.model
            .find({ ...query, isDeleted: false })
            .skip(skip)
            .limit(limit)
            .sort({ createdAt: -1 })

        return {
            data: data,
            pageCount: pageCount,
            total: total
        };
    }

    async deleteEntity(id: string): Promise<boolean> {
        const data = await this.model.findByIdAndUpdate(
            id,
            { isDeleted: true },
            { new: true }
        );
        if (data) return true;
        return false;
    }
    
    async findOneByFilterQuery(query: any): Promise<CategoryDetails | null> {
        return await this.model.find({ ...query, isDeleted: false }).lean();
    }

    async findAll(): Promise<CategoryDetails[]> {
        return await this.model.find({ isDeleted: false });
    }
}