import mongoose, { Model, Types } from 'mongoose';
import { CommonFormSubmit } from './schema/common-form-submit.schema';
import { CreateCommonFormSubmitDto } from './dto/create-common-form-submit.dto';
import { UpdateCommonFormSubmitDto } from './dto/update-common-form-submit.dto';
import * as moment from 'moment';


export class CommonFormSubmitRepository<CommonFormSubmitDocument extends CommonFormSubmit> {
    constructor(private readonly model: Model<CommonFormSubmitDocument>) { }

    async createEntity(data: CreateCommonFormSubmitDto): Promise<CommonFormSubmit> {
        const createdEntity = new this.model({
            ...data,
            _id: new Types.ObjectId()
        });
        return await createdEntity.save();
    }

    async findOneEntity(id: string): Promise<CommonFormSubmit | null> {
        if (!mongoose.Types.ObjectId.isValid(id)) {
            return null;
        }
        
        return await this.model.findOne({ _id: id, isDeleted: false });
    }

    async findOneEntityBySlug(urlSlug: string): Promise<CommonFormSubmit | null> {
        try {
            return await this.model.findOne({ urlSlug: urlSlug, isDeleted: false });
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, data: UpdateCommonFormSubmitDto): Promise<CommonFormSubmit | null> {
        return await this.model.findByIdAndUpdate(id, data, { new: true });
    }

    async paginate(page: number, limit: number, query?: any) {
        const skip = (page - 1) * limit;
        const total = await this.model.countDocuments({ ...query, isDeleted: false });
        const pageCount = Math.ceil(total / limit);

        const data = await this.model
            .find({ ...query, isDeleted: false })
            .skip(skip)
            .limit(limit)
            .sort({ createdAt: -1 })

        return {
            data: data,
            pageCount: pageCount,
            total: total
        };
    }

    async deleteEntity(id: string): Promise<boolean> {
        const data = await this.model.findByIdAndUpdate(
            id,
            { isDeleted: true },
            { new: true }
        );
        if (data) return true;
        return false;
    }

    async findAll(): Promise<CommonFormSubmit[]> {
        return await this.model.find({ isDeleted: false });
    }

    async countAllQuery(): Promise<any | Error> {
        try {
            return await this.model.countDocuments({
                isDeleted: false
            });
        } catch (e) {
            return e;
        }
    }

    async countCompletedDocumentsByMonth(): Promise<any> {
        const currentDate = moment();
        const startOfMonthDate = currentDate.startOf('month');
        const endOfMonthDate = currentDate.endOf('month');

        // Calculate the start and end dates for the previous 6 months
        const startOfLast12Months = startOfMonthDate.clone().subtract(6, 'months');
        const endOfLast12Months = startOfMonthDate.clone().subtract(1, 'months');

        // Generate a list of months you want to display for the last 6 months
        const monthsList = [];
        let currentMonth = startOfLast12Months.clone();
        while (currentMonth.isSameOrBefore(endOfMonthDate, 'month')) {
            monthsList.push(currentMonth.format('YYYY-MM'));
            currentMonth.add(1, 'month');
        }
        return await this.model.aggregate([
            {
                $match: {
                    isDeleted: false,
                    createdAt: {
                        $gte: startOfLast12Months.toDate(), // Documents with createdAt >= start of last 12 months
                        $lte: endOfMonthDate.toDate(), // Documents with createdAt <= end of the current month
                    },
                },
            },
            {
                $group: {
                    _id: {
                        year: { $year: '$createdAt' },
                        month: { $month: '$createdAt' },
                    },
                    count: { $sum: 1 },
                },
            },
            {
                $sort: { '_id.year': 1, '_id.month': 1 },
            },
            {
                $project: {
                    year: '$_id.year',
                    month: {
                        $switch: {
                            branches: [
                                { case: { $eq: ['$_id.month', 1] }, then: 'January' },
                                { case: { $eq: ['$_id.month', 2] }, then: 'February' },
                                { case: { $eq: ['$_id.month', 3] }, then: 'March' },
                                { case: { $eq: ['$_id.month', 4] }, then: 'April' },
                                { case: { $eq: ['$_id.month', 5] }, then: 'May' },
                                { case: { $eq: ['$_id.month', 6] }, then: 'June' },
                                { case: { $eq: ['$_id.month', 7] }, then: 'July' },
                                { case: { $eq: ['$_id.month', 8] }, then: 'August' },
                                { case: { $eq: ['$_id.month', 9] }, then: 'September' },
                                { case: { $eq: ['$_id.month', 10] }, then: 'October' },
                                { case: { $eq: ['$_id.month', 11] }, then: 'November' },
                                { case: { $eq: ['$_id.month', 12] }, then: 'December' },
                            ],
                            default: 'Unknown',
                        },
                    },
                    count: 1,
                },
            },
            {
                $project: { _id: 0 } // Exclude the _id field
            },
        ])
            .then((result) => {
                // Create a map for easier access to the aggregated data
                const resultMap = new Map(
                    result.map((item) => {
                        const monthKey = moment(`${item.year}-${item.month}`, 'YYYY-MMM').format('YYYY-MM');
                        return [monthKey, item];
                    })
                );
                // Create the final output including months with count 0
                const finalOutput = monthsList.map((month) => {
                    const matchingResult = resultMap.get(month);
                    const [year, monthName] = month.split('-');
                    return {
                        year: parseInt(year, 10),
                        month: matchingResult ? matchingResult.month : moment(`${year}-${monthName}`, 'YYYY-MM').format('MMMM'),
                        count: matchingResult ? matchingResult.count : 0,
                    };
                });

                return finalOutput;
            });
    }

    async countDocumentsByFilterQuery(query: any) {
        return await this.model.countDocuments({ isDeleted: false, ...query });
    }
}