import { BadRequestException, Injectable, NotFoundException } from '@nestjs/common';
import { InjectModel } from '@nestjs/mongoose';
import { Model } from 'mongoose';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { CommonFormSubmitRepository } from './common-form-submit.repository';
import { CommonFormSubmit } from './schema/common-form-submit.schema';
import { CreateCommonFormSubmitDto } from './dto/create-common-form-submit.dto';
import { UpdateCommonFormSubmitDto } from './dto/update-common-form-submit.dto';
import { ContactPageCmsRepository } from 'src/modules/CMS/contact-page-cms/contact-page-cms.repository';
import { ContactPageCms } from 'src/modules/CMS/contact-page-cms/schema/contact-page-cms.schema';
import { MailSenderService } from 'src/modules/mail-sender/mail-sender.service';
import { CreateCommonContactFormMailSenderDto } from 'src/modules/mail-sender/dto/create-common-contact-form-mail.dto';


@Injectable()
export class CommonFormSubmitService {
    constructor(
        private readonly doSpaceService: DoSpacesService,
        @InjectModel('common-form-submit') private readonly model: Model<CommonFormSubmit>,
        @InjectModel('contact_page_cms') private readonly contactCmsModel: Model<ContactPageCms>,
        private readonly mailService: MailSenderService
    ) { }

    private readonly repository = new CommonFormSubmitRepository(this.model);
    private readonly contactPageCmsRepository = new ContactPageCmsRepository(this.contactCmsModel);

    async create(
        dto: CreateCommonFormSubmitDto
    ): Promise<CommonFormSubmit> {

        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new BadRequestException(
                Constants.CREATE_FAILED
            );
        }

        // Email template start
        const contactData = await this.contactPageCmsRepository.findAll();

        const mailDto = {
            to: data?.email,
            from: process.env.EMAIL_SENDER_MAIL,
            subject: '1-800-Pain-800 center for pain control',
            template: './templates/common_form_submit_email',
            companyEmail: contactData?.[0]?.email,
            companyAddress: contactData?.[0]?.address,
            companyNumber: contactData?.[0]?.contactNumber,
            customerName: data?.firstName + ' ' + data?.lastName,
            customerNumber: data?.contactNumber,
            customerEmail: data?.email,
            customerMessage: data?.message
        };

        await this.mailService.commonContactFormMailSender(mailDto as CreateCommonContactFormMailSenderDto);
        // Email template end

        return ResponseUtils.successResponseHandler(201, "Successfully added data.", "data", data);
    }

    async findAll(pageParam?: string, limitParam?: string): Promise<CommonFormSubmit[]> {
        const page = parseInt(pageParam, 10) ? parseInt(pageParam, 10) : 1;
        const limit = parseInt(limitParam, 10) ? parseInt(limitParam, 10) : 10;
        const data = await this.repository.paginate(page, limit);
        if (!data) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<CommonFormSubmit | null> {
        const data = await this.repository.findOneEntity(id);
        if (!data) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async update(
        id: string, 
        dto: UpdateCommonFormSubmitDto
    ): Promise<CommonFormSubmit | null> {
        const res = await this.repository.findOneEntity(id);
        if (!res) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }

        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new BadRequestException(
                Constants.UPDATE_FAILED
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }

    async delete(id: string): Promise<void> {
        const res = await this.repository.findOneEntity(id);
        if (!res) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }

        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new BadRequestException(
                Constants.DELETE_FAILED
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}