import { BadRequestException, Injectable, NotFoundException } from '@nestjs/common';
import { InjectModel } from '@nestjs/mongoose';
import { Model } from 'mongoose';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { SocialLinks } from './schema/social-links.schema';
import { SocialLinksRepository } from './social-links.repository';
import { CreatSocialLinksDto } from './dto/create-social-links.dto';
import { UpdateSocialLinksDto } from './dto/update-social-links.dto';


@Injectable()
export class SocialLinksService {
    constructor(
        private readonly doSpaceService: DoSpacesService,
        @InjectModel('social-links') private readonly model: Model<SocialLinks>
    ) { }

    private readonly repository = new SocialLinksRepository(this.model);

    async create(
        dto: CreatSocialLinksDto,
        files: {
            image?: UploadedMulterFileI
        }
    ): Promise<SocialLinks> {
        if (files && files.image) {
            const img: any = await this.doSpaceService.uploadFile(files.image[0], "social-links/");
            dto.image = img;
        }

        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new BadRequestException(
                Constants.CREATE_FAILED
            );
        }
        return ResponseUtils.successResponseHandler(201, "Successfully added to Social Links.", "data", data);
    }

    async findAll(pageParam?: string, limitParam?: string): Promise<SocialLinks[]> {
        const page = parseInt(pageParam, 10) ? parseInt(pageParam, 10) : 1;
        const limit = parseInt(limitParam, 10) ? parseInt(limitParam, 10) : 10;
        const data = await this.repository.paginate(page, limit);
        if (!data) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<SocialLinks | null> {
        const data = await this.repository.findOneEntity(id);
        if (!data) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async update(
        id: string, 
        dto: UpdateSocialLinksDto,
        files: {
            image?: UploadedMulterFileI
        }
    ): Promise<SocialLinks | null> {
        const res = await this.repository.findOneEntity(id);
        if (!res) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }
        const foundImage = (res as SocialLinks)?.image;
        if (files && files.image) {
            const img: any = await this.doSpaceService.uploadFile(files.image[0], "social-links/");
            dto.image = img;
        } else {
            dto.image = foundImage;
        }

        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new BadRequestException(
                Constants.UPDATE_FAILED
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }

    async delete(id: string): Promise<void> {
        const res = await this.repository.findOneEntity(id);
        if (!res) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }

        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new BadRequestException(
                Constants.DELETE_FAILED
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}