import { BadRequestException, Injectable, NotFoundException } from '@nestjs/common';
import { InjectModel } from '@nestjs/mongoose';
import { Model } from 'mongoose';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { Testimonial } from './schema/testimonial.schema';
import { TestimonialRepository } from './testimonial.repository';
import { CreateTestimonialDto } from './dto/create-testimonial.dto';
import { UpdateTestimonialDto } from './dto/update-testimonial.dto';



@Injectable()
export class TestimonialService {
    constructor(
        private readonly doSpaceService: DoSpacesService,
        @InjectModel('testimonial') private readonly model: Model<Testimonial>
    ) { }

    private readonly repository = new TestimonialRepository(this.model);

    async create(
        dto: CreateTestimonialDto,
        files: {
            userImage?: UploadedMulterFileI
        }
    ): Promise<Testimonial> {
        if (files && files.userImage) {
            const img: any = await this.doSpaceService.uploadFile(files.userImage[0], "Testimonial/");
            dto.userImage = img;
        }

        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new BadRequestException(
                Constants.CREATE_FAILED
            );
        }
        return ResponseUtils.successResponseHandler(201, "Successfully added to Testimonial.", "data", data);
    }

    async findAll(pageParam?: string, limitParam?: string): Promise<Testimonial[]> {
        const page = parseInt(pageParam, 10) ? parseInt(pageParam, 10) : 1;
        const limit = parseInt(limitParam, 10) ? parseInt(limitParam, 10) : 10;
        const data = await this.repository.paginate(page, limit);
        if (!data) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<Testimonial | null> {
        const data = await this.repository.findOneEntity(id);
        if (!data) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async update(
        id: string, 
        dto: UpdateTestimonialDto,
        files: {
            userImage?: UploadedMulterFileI
        }
    ): Promise<Testimonial | null> {
        const res = await this.repository.findOneEntity(id);
        if (!res) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }
        const foundImage = (res as Testimonial)?.userImage;
        if (files && files.userImage) {
            const img: any = await this.doSpaceService.uploadFile(files.userImage[0], "Testimonial/");
            dto.userImage = img;
        } else {
            dto.userImage = foundImage;
        }

        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new BadRequestException(
                Constants.UPDATE_FAILED
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }

    async delete(id: string): Promise<void> {
        const res = await this.repository.findOneEntity(id);
        if (!res) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }

        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new BadRequestException(
                Constants.DELETE_FAILED,
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}