import mongoose, { Model, Types } from 'mongoose';
import CryptoUtils from 'src/utils/crypto.utils';
import { CreateUsersDto } from './dto/create-users.dto';
import { UpdateUsersDto } from './dto/update-users.dto';
import { Users } from './schema/users.schema';

export class UsersRepository<UsersDocument extends Users> {
    constructor(private readonly model: Model<UsersDocument>) { }

    async createEntity(data: CreateUsersDto): Promise<Users> {
        const createdEntity = new this.model({
            ...data,
            password: await CryptoUtils.encrypt(data?.password),
            _id: new Types.ObjectId()
        });
        return await createdEntity.save();
    }

    async findOneEntity(id: string): Promise<Users | null> {
        if (!mongoose.Types.ObjectId.isValid(id)) {
            return null;
        }

        return await this.model.findOne({ _id: id, isDeleted: false });
    }

    async findOneByFilterQuery(query: any): Promise<Users | null> {
        return await this.model.findOne({ ...query, isDeleted: false }).lean();
    }
    async findOneWhichDelete(query: any): Promise<Users | null> {
        return await this.model.findOne({ ...query }).lean();
    }

    async deleteForeverEntity(id: string): Promise<boolean> {
        const data = await this.model.deleteOne({ _id: id });
        if (data) return true;
        return false;
    }

    async updateEntity(id: string, data: UpdateUsersDto): Promise<Users | null> {
        const updatedEntity = await this.model.findByIdAndUpdate(id, data, {
            new: true
        });
        const leanEntity = updatedEntity.toObject({ getters: true });

        return leanEntity;
    }

    async deleteEntity(id: string): Promise<boolean> {
        const data = await this.model.findByIdAndUpdate(
            id,
            { isDeleted: true },
            { new: true }
        );
        if (data) return true;
        return false;
    }

    async findAll(): Promise<Users[]> {
        return await this.model.find({ isDeleted: false });
    }

    async paginate(page: number, limit: number, query?: any) {
        const skip = (page - 1) * limit;
        const total = await this.model.countDocuments({
            ...query,
            isDeleted: false
        });
        const pageCount = Math.ceil(total / limit);

        const data = await this.model
            .find({ ...query, isDeleted: false })
            .skip(skip)
            .limit(limit)
            .sort({ createdAt: -1 });

        return {
            data: data,
            pageCount: pageCount,
            total: total
        };
    }

    async countDocumentsByFilterQuery(query: any) {
        return await this.model.countDocuments({ isDeleted: false, ...query });
    }

    async findByFilterQuery(query: any): Promise<Users[] | null> {
        return await this.model.find({ ...query, isDeleted: false }).lean();
    }
}
