import {
    BadRequestException,
    Injectable,
    NotFoundException,
} from '@nestjs/common';
import { InjectModel } from '@nestjs/mongoose';
import { Model } from 'mongoose';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { CreateMailSenderDto } from 'src/modules/mail-sender/dto/create-mail-sender.dto';
import { MailSenderService } from 'src/modules/mail-sender/mail-sender.service';
import { Constants } from 'src/utils/constants';
import CryptoUtils from 'src/utils/crypto.utils';
import { ResponseUtils } from 'src/utils/response.utils';
import { v4 as uuidv4 } from 'uuid';
import { TemporaryPassword } from '../temporary-password/schema/temporary-password.schema';
import { TemporaryPasswordRepository } from '../temporary-password/temporary-password.repository';
import { CreateUsersDto } from './dto/create-users.dto';
import { UpdateUsersDto } from './dto/update-users.dto';
import { Users } from './schema/users.schema';
import { UsersRepository } from './users.repository';

@Injectable()
export class UsersService {
    constructor(
        private readonly doSpaceService: DoSpacesService,
        @InjectModel('users') private readonly usersModel: Model<Users>,
        @InjectModel('temporary_password') private readonly temporaryPasswordModel: Model<TemporaryPassword>,
        private readonly mailService: MailSenderService
    ) { }

    private readonly usersRepository = new UsersRepository(this.usersModel);
    private readonly temporaryPasswordRepository = new TemporaryPasswordRepository(this.temporaryPasswordModel);

    async create(
        dto: CreateUsersDto,
        files: {
            image?: UploadedMulterFileI;
        }
    ): Promise<Users> {
        try {
            if (files && files.image) {
                const image: any = await this.doSpaceService.uploadFile(
                    files.image[0],
                    'users/'
                );
                dto.image = image;
            } else dto.image = 'https://neutrix-site.nyc3.digitaloceanspaces.com/default.png';

            if (!dto?.userType) {
                dto.password = uuidv4().replace(/-/g, '').substring(0, 6);
            }
            const res = await this.usersRepository.findOneByFilterQuery({
                email: dto.email
            });
            if (res) {
                throw new BadRequestException(`User already exists!`);
            }
            const response = await this.usersRepository.findOneWhichDelete({
                email: dto.email
            });
            if (response) {
                const data = await this.usersRepository.deleteForeverEntity(
                    response._id.toString()
                );
                if (!data) {
                    throw new BadRequestException(Constants.DELETE_FAILED);
                }
            }

            const data = await this.usersRepository.createEntity(dto);

            if (!data) {
                throw new BadRequestException(Constants.CREATE_FAILED);
            }

            if (data?.userType === 'agent') {
                await this.temporaryPasswordRepository.createEntity({
                    userId: data?._id,
                    temporary_password: dto.password
                });

                // const mailDto = {
                //     to: data?.email,
                //     from: process.env.EMAIL_SENDER_MAIL,
                //     subject: 'L.A. Agent Account Creation Notification',
                //     template: './templates/agent_welcome_email',
                //     username: data?.firstName + ' ' + data?.lastName,
                //     password: dto.password,
                //     email: data?.email,
                //     privacyPolicyPage: process.env.PRIVACY_POLICY_PAGE,
                //     termsOfUsePage: process.env.TERMS_OF_USE_PAGE,
                //     loginPage: process.env.LOGIN_PAGE
                // };

                // await this.mailService.agentAccountCreateMailSender(
                //     mailDto as CreateMailSenderDto
                // );
            }

            return ResponseUtils.successResponseHandler(201, 'Successfully created data', 'data', data);
        } catch (err) {
            throw new BadRequestException(err);
        }
    }

    async findAll(
        pageParam: string,
        limitParam: string,
        userType: string
    ): Promise<Users[]> {
        let data: any;
        const query: any = {};
        if (userType) query.userType = userType;

        if (pageParam && limitParam) {
            const page = parseInt(pageParam, 10) ? parseInt(pageParam, 10) : 1;
            const limit = parseInt(limitParam, 10) ? parseInt(limitParam, 10) : 10;
            data = await this.usersRepository.paginate(page, limit, query);
        } else if (userType) {
            data = await this.usersRepository.findByFilterQuery(query);
        } else {
            data = await this.usersRepository.findAll();
        }

        if (!data) {
            throw new NotFoundException(Constants.NOT_FOUND);
        }
        return ResponseUtils.successResponseHandler(
            200,
            'Data fetched successfully',
            'data',
            data
        );
    }

    async findOne(id: string): Promise<Users | null> {
        const data = await this.usersRepository.findOneEntity(id);
        if (!data) {
            throw new NotFoundException(Constants.NOT_FOUND);
        }
        return ResponseUtils.successResponseHandler(
            200,
            'Data fetched successfully',
            'data',
            data
        );
    }

    async update(
        id: string,
        dto: UpdateUsersDto,
        files: {
            image?: UploadedMulterFileI;
        }
    ): Promise<Users | null> {
        const res = await this.usersRepository.findOneEntity(id);
        if (!res) {
            throw new NotFoundException(Constants.NOT_FOUND);
        }
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(
                files.image[0],
                'users/'
            );
            dto.image = image;
        } else if (res?.image) dto.image = res?.image;
        else dto.image = 'https://neutrix-site.nyc3.digitaloceanspaces.com/default.png';

        const { password, ...updateDto } = dto;
        const updatedData = await this.usersRepository.updateEntity(id, updateDto);
        if (!updatedData) {
            throw new BadRequestException(Constants.UPDATE_FAILED);
        }
        const data = {
            ...updatedData,
            password: undefined
        };
        return ResponseUtils.successResponseHandler(
            200,
            'Data updated successfully',
            'data',
            data
        );
    }

    async changePassword(
        currentPassword: string,
        newPassword: string,
        userId: string
    ): Promise<Users | null> {
        const user = await this.usersRepository.findOneEntity(userId);
        const compare = await CryptoUtils.compare(currentPassword, user.password);

        if (!compare) {
            throw new NotFoundException('Password does not match!');
        }
        await this.usersRepository.updateEntity(userId, {
            password: await CryptoUtils.encrypt(newPassword)
        });

        return ResponseUtils.successResponseHandler(
            200,
            'Password changed successfully!',
            'data',
            ''
        );
    }

    async delete(id: string): Promise<void> {
        const res = await this.usersRepository.findOneEntity(id);
        if (!res) {
            throw new NotFoundException(Constants.NOT_FOUND);
        }

        const data = await this.usersRepository.deleteEntity(id);
        if (!data) {
            throw new BadRequestException(Constants.DELETE_FAILED);
        }
        return ResponseUtils.buildDeletedData(
            data,
            200,
            'Data deleted successfully'
        );
    }
}
