import {
    BadRequestException,
    HttpException,
    HttpStatus,
    Injectable,
    InternalServerErrorException,
    NotFoundException,
    UnauthorizedException
} from '@nestjs/common';
import { ResponseUtils } from 'src/utils/response.utils';
import { v4 as uuidv4 } from 'uuid';
import JwtHelper from '../../core/jwt/jwt.helper';
import { Constants } from '../../utils/constants';
import { LoginAuthDto } from './dto/login-auth.dto';
import { RegisterAuthDto } from './dto/register-auth.dto';
import { InjectModel } from '@nestjs/mongoose';
import { Model } from 'mongoose';
import { Users } from '../CRUD/users/schema/users.schema';
import { UsersRepository } from '../CRUD/users/users.repository';
import CryptoUtils from 'src/utils/crypto.utils';
import { ResetPasswordRequestDto } from './dto/reset-password.dto';
import { ChangePasswordToken } from '../CRUD/change-password-token/schema/change-password-token.schema';
import { ChangePasswordTokenRepository } from '../CRUD/change-password-token/change-password-token.repository';
import { ResetPasswordMailDto } from '../mail-sender/dto/create-reset-password-mail.dto';
import { MailSenderService } from '../mail-sender/mail-sender.service';
import { CheckVerifyHashPasswordDto } from './dto/check-verify-hash-password-token';
import { UpdatePasswordUsingVerifyHashPasswordTokenDto } from './dto/update-password-using-verify-hash-password-token';

@Injectable()
export class AuthService {
    constructor(
        private readonly jwtHelper: JwtHelper,
        private readonly servMail: MailSenderService,
        @InjectModel('Users') private readonly usersModel: Model<Users>,
        @InjectModel('change_password_token') private readonly changePasswordTokenModel: Model<ChangePasswordToken>
    ) { }

    private readonly usersRepository = new UsersRepository(this.usersModel);
    private readonly changePasswordTokenRepository = new ChangePasswordTokenRepository(this.changePasswordTokenModel);

    async login(dto: LoginAuthDto): Promise<any> {
        const { email, password } = dto;
        const data = await this.usersRepository.findOneByFilterQuery({ email: email })

        if (!data) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }

        const resData = await this.authenticateUser(password, data)
        if (resData == null) {
            throw new UnauthorizedException(
                Constants.UNAUTH_REQ
            );
        }
        return resData;
    }

    async authenticateUser(
        password: string,
        user: Users
    ): Promise<any> {
        const { password: passwordHash, ...entityWithoutPassword } = user;

        const matched = await CryptoUtils.compare(password, passwordHash);
        if (matched) {
            const token = this.jwtHelper.generateToken(
                {
                    userId: entityWithoutPassword._id
                },
                '7d'
            );
            return {
                token: token,
                user: entityWithoutPassword
            };
        }
        return null;
    }

    async register(dto: RegisterAuthDto): Promise<Users | Error> {
        const data = await this.usersRepository.createEntity(dto);

        if (!data) {
            throw new BadRequestException(
                Constants.CREATE_FAILED
            );
        }

        const { email, password } = dto;
        const loginData = await this.login({
            email: email,
            password: password
        });
        return ResponseUtils.successResponseHandler(201, "Customer registered successfully!", "data", loginData);
    }

    async validateRequest(req: any): Promise<boolean> {
        try {
            const bearerToken = req.headers.authorization;
            const bearer = 'Bearer ';
            if (!bearerToken || !bearerToken.startsWith(bearer)) {
                return false;
            }
            const token = bearerToken.replace(bearer, '');
            const jwtPayload = this.jwtHelper.verifyToken(token);
            const data = await this.usersRepository.findOneEntity(jwtPayload.userId);
            if (!data || data instanceof Error) {
                return false;
            }
            req.data = data;
            return true;
        } catch (e) {
            return false;
        }
    }

    // async forgotPassword(dto: ResetPasswordRequestDto): Promise<Users | Error> {
    //     let userCheck = await this.usersRepository.findOneByFilterQuery({ email: dto.email });
    //     if (!userCheck) {
    //         throw new NotFoundException(
    //             Constants.NOT_FOUND
    //         );
    //     }

    //     let payload = {
    //         email: dto.email,
    //         token: uuidv4(),
    //         expired: false
    //     };

    //     let data = await this.changePasswordTokenRepository.createEntity(payload);

    //     let mailDto = {
    //         "to": [
    //             dto.email
    //         ],
    //         "from": process.env.EMAIL_SENDER_MAIL,
    //         "subject": "L.A. Insurance password reset request",
    //         "username": `${userCheck?.firstName} ${userCheck?.lastName}`,
    //         "token": payload.token,
    //         "email": dto.email
    //     }

    //     this.servMail.sendPasswordResetRequestMail(mailDto as ResetPasswordMailDto);

    //     return ResponseUtils.buildData(true);
    // }

    async verifyHashPasswordTokenRequest(dto: CheckVerifyHashPasswordDto) {
        let userCheck = await this.changePasswordTokenRepository.findOneByFilterQuery({ token: dto.hashPass, expired: false });
        if (!userCheck) {
            return ResponseUtils.buildData(false);
        } else {
            return ResponseUtils.buildData(true);
        }
    }

    async updatePasswordUsingTokenRequest(
        dto: UpdatePasswordUsingVerifyHashPasswordTokenDto
    ) {
        let userCheck = await this.changePasswordTokenRepository.findOneByFilterQuery({ token: dto.passToken, expired: false });
        if (!userCheck) {
            return ResponseUtils.buildData(false);
        } else {
            let userInfo = await this.usersRepository.findOneByFilterQuery({ email: userCheck.email });
            if (!userInfo) {
                throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
            }
            let updatePassword = await this.usersRepository.updateEntity(userInfo._id.toString(), {
                password: await CryptoUtils.encrypt(dto.password)
            })
            if (updatePassword) {
                await this.changePasswordTokenRepository.updateEntity(userCheck._id.toString(), {
                    expired: true
                })
            }
            return ResponseUtils.buildData(true);
        }
    }
}
